import java.awt.Color;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.util.Random;
import java.util.concurrent.TimeoutException;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManagerFactory;
import javax.swing.JTextArea;

import com.rabbitmq.client.AuthenticationFailureException;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;

public class AMQPConnection {
// this class handles the technical connection to the M7 API servers

	private String userName 	= "";
	private String password 	= "";
	private String rabbitHost 	= "";
	private String rabbitHost1 	= "";
	private String rabbitHost2 	= "";
	private int    rabbitPort	= 0;
	private String rabbitVHost	= "";
	private String certPath 	= "";
	private String certPassword = "";
	private String applicationID = "";
	private String credentials 	= "";
	public int connectedURLNb = 0;
	public Connection connection = null; 
	public int connectionLossNb = 0;
	private Log appLog = null;
	private ControlBoard controlBoard = null;
	
	private final int AMQP_HEARTBEAT_INTERVAL= 10 ; // in seconds
	
	// ***************************************************
	// *** Constructor
	// ***************************************************	
	public AMQPConnection (String userName, String password, String rabbitHost1, String rabbitHost2, int rabbitPort, 
							String rabbitVHost, String certPath, String certPassword, String applicationID, ControlBoard controlBoard, Log appLog) {
		
		this.userName 	= userName;
		this.password 	= password;
		this.rabbitHost1 = rabbitHost1;
		this.rabbitHost2 = rabbitHost2;
		this.rabbitVHost = rabbitVHost;
		this.rabbitPort = rabbitPort;
		this.certPath 	 = certPath;
		this.certPassword = certPassword;
		this.applicationID = applicationID;
		this.controlBoard = controlBoard;
		this.appLog = appLog;
		
		this.credentials ="userName='"+userName + "'" 
					+ "; password='"+password + "'"
					+ "; rabbitHost1='"+rabbitHost1 + "'"
					+ "; rabbitHost2='"+rabbitHost2 + "'"
					+ "; rabbitPort='" + rabbitPort + "'"
					+ "; rabbitVHost='"+ rabbitVHost + "'"
					+ "; applicationID='"+applicationID + "'"
					+ "; certPath='"+certPath + "'"
					+ ";certPassword="+certPassword ;
	}

	// ***************************************************
	// *** Connect implementing the Client Failover (see DFS180)
	// ***************************************************
	public void ConnectWithClientFailover (ControlBoard controlBoardFrame) {
		
		int URLNb = 0;
		
		// randomly select end point 1 or 2 (rabbit Host)
		Random randomP = new Random();
	    int endPointNb = randomP.nextInt(2)+1;
	    
	    if (endPointNb ==1) {
	    	this.rabbitHost = this.rabbitHost1;
			URLNb = 1;
	    }
	    else { 
	    	rabbitHost = this.rabbitHost2;
			URLNb = 2;
	    }
	    
	    controlBoard.AddText(this.credentials,controlBoard.textAreaConnection);
		appLog.logger.info(this.credentials);
		
		String msg = "endPointNb =" + endPointNb + " chosen host =" + this.rabbitHost;
		controlBoard.AddText(msg,controlBoard.textAreaConnection);
	    appLog.logger.info(msg);
		
		// prepare the AMQP connection
		ConnectionFactory factory = new ConnectionFactory();
		
		factory.setAutomaticRecoveryEnabled(false); // important to avoid gaps (because network hickups are hidden and messages lost, see M7 API FAQ)
		factory.setRequestedHeartbeat(this.AMQP_HEARTBEAT_INTERVAL);
		factory.useSslProtocol(createSslContext(this.certPath, this.certPassword));
		
		factory.setUsername(this.userName);
		factory.setPassword(this.password);
		factory.setHost(this.rabbitHost);
		factory.setPort(this.rabbitPort);
		factory.setVirtualHost(this.rabbitVHost);

		// Connect to the AMQP server/broker
		try {  
			msg ="Trying to connect to the AMQP server " + endPointNb;
			controlBoard.AddText(msg,controlBoard.textAreaConnection);
			appLog.logger.info(msg);
			
			connection = factory.newConnection();
			this.connectedURLNb = URLNb;
		}
		catch (TimeoutException e) {
			e.printStackTrace();
			controlBoard.AddText(e.toString(),controlBoard.textAreaConnection);
			msg = "!!! Timeout. Could not establish a connection to the AMQP server with above parameters !!!";
			appLog.logger.info(msg);
			controlBoard.AddText(msg,controlBoard.textAreaConnection);
		}  
		catch (AuthenticationFailureException e) {
			e.printStackTrace();
			controlBoard.AddText(e.toString(),controlBoard.textAreaConnection);
			msg = "!!! AuthenticationFailureException. Could not establish a connection to the AMQP server with above parameters !!!";
			appLog.logger.info(msg);
			controlBoard.AddText(msg,controlBoard.textAreaConnection);
		} 
		catch (Exception e) {
			e.printStackTrace();
			controlBoard.AddText(e.toString(),controlBoard.textAreaConnection);
			msg = "!!! Exception. Could not establish a connection to the AMQP server with above parameters !!!";
			appLog.logger.info(msg);
			controlBoard.AddText(msg,controlBoard.textAreaConnection);
		}  
		
		// if the connection could not be established, try to connect to the other URL right away		
		if (connection == null) {
			// select the other end point
			if (rabbitHost.contentEquals(rabbitHost1)) {
					rabbitHost = rabbitHost2;
					URLNb = 2;
			}
			else {
				rabbitHost = rabbitHost1;
				URLNb = 1;
			}
			
			try {
				// Connect to the AMQP server/broker
				msg = "Trying to connect to the other AMQP server";
				controlBoard.AddText(msg,controlBoard.textAreaConnection);
				appLog.logger.info(msg);
				
				factory.setHost(rabbitHost);
				
				connection = factory.newConnection();
				this.connectedURLNb = URLNb;
			}
			catch (TimeoutException e) {
				e.printStackTrace();
				controlBoard.AddText(e.toString(),controlBoard.textAreaConnection);
				msg = "!!! Timeout. Could not establish a connection to the AMQP server with above parameters !!!";
				appLog.logger.info(msg);
				controlBoard.AddText(msg,controlBoard.textAreaConnection);
			}  
			catch (AuthenticationFailureException e) {
				e.printStackTrace();
				controlBoard.AddText(e.toString(),controlBoard.textAreaConnection);
				msg = "!!! AuthenticationFailureException. Could not establish a connection to the AMQP server with above parameters !!!";
				appLog.logger.info(msg);
				controlBoard.AddText(msg,controlBoard.textAreaConnection);
			} 
			catch (Exception e) {
				e.printStackTrace();
				controlBoard.AddText(e.toString(),controlBoard.textAreaConnection);
				msg = "!!! Exception. Could not establish a connection to the AMQP server with above parameters !!!";
				appLog.logger.info(msg);
				controlBoard.AddText(msg,controlBoard.textAreaConnection);
			}  
			
			if (connection == null) {
				// wait for 30 seconds before reconnecting ; should implement here an exponential back off
				try {
					msg = "Waiting for 30 seconds before trying again to connect...";
					controlBoard.AddText(msg,controlBoard.textAreaConnection);
					controlBoard.AddText("",controlBoard.textAreaConnection);
					Thread.sleep(30000);
				} catch (InterruptedException e) {
					e.printStackTrace();
					controlBoard.AddText(e.toString(),controlBoard.textAreaConnection);
					appLog.logger.info(e.toString());
				}
				//try again to connect
				this.ConnectWithClientFailover(controlBoardFrame);
			}
			else {
				controlBoardFrame.successfulConnection(this);
			}
		}
		else {
			controlBoardFrame.successfulConnection(this);
		}
		
		msg = "connection isOpen= "+connection.isOpen()+"\n";
		appLog.logger.info(msg);
		controlBoard.AddText(msg,controlBoard.textAreaConnection);
		
	}

	// **************************************
	//* ConnectionIsOpen
	// **************************************!
	public boolean ConnectionIsOpen () {
		if (this.connection != null) {
			return this.connection.isOpen();			
		}
		else return false;
	}
	
	// **************************************
	//* createSslContext
	// **************************************!
	private SSLContext createSslContext(String certPath, String certificatePassword) {

		try {

			KeyStore ks = KeyStore.getInstance("PKCS12");
			InputStream certificate =  new FileInputStream(new File(certPath));

			char[] certificatePasswordChar = certPassword.toCharArray();

			ks.load(certificate, certificatePasswordChar);

			String algorithmX509 = KeyManagerFactory.getDefaultAlgorithm();

			KeyManagerFactory kmf = KeyManagerFactory.getInstance(algorithmX509);

			kmf.init(ks, certificatePasswordChar);
			SSLContext ctx = SSLContext.getInstance("TLSv1.2");

			TrustManagerFactory trustManagerFactory = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());

			trustManagerFactory.init((KeyStore) null);

			ctx.init(kmf.getKeyManagers(), trustManagerFactory.getTrustManagers(), null);

			return ctx;

		} catch (GeneralSecurityException | IOException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	// *********************
	// closeConnection
	// *********************
	public void closeConnection(String context) {
		appLog.logger.info("\n\n"+"***** closeConnection() ***"+"\n\n");
		appLog.logger.info("AMQP connection check");
		if (this.connection != null) {
			try {
				appLog.logger.info("Connection not null: attempt to close the AMQP connection in context:" + context);
				this.connection.close();
				appLog.logger.info("Connection closed."+context);
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
				if (! this.connection.isOpen()) {
					appLog.logger.warning("attempt to close the AMQP connection");
				} else {
					appLog.logger.warning("Unable to close current connection"+ e1);
				}
			}
		}
		else {
			appLog.logger.info("AMQP connection is null. Not closed.");
		}
	}

}
