import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

public class BroadcastList {
	private CopyOnWriteArrayList<Broadcast> broadcastList = new CopyOnWriteArrayList<>();
	private SimpleDateFormat sdf = new SimpleDateFormat("yyy/MM/dd HH:mm:ss.sss") ;
	private int nbBroadcasts = 0;
	private final static String BROADCAST_TYPE_PREFIX = "com.deutscheboerse.m7.trading.api.v6.";
	private boolean alreadySortedOnce = false;
	private ControlBoard controlBoardFrame;
	
	private final static String FILE_BROADCAST_LIST_NAME = "Broadcasts.csv";
	private final static String NEW_LINE_SEPARATOR ="\n";
	private final static String CSV_SEPARATOR =";";
	private final static String FILE_BROADCAST_LIST_HEADER = 
														"Client Reception Time"+CSV_SEPARATOR
														+ "AMQP Sending Time"+CSV_SEPARATOR 
														+ "Type"+CSV_SEPARATOR
														+ "Correlation Id"+CSV_SEPARATOR
														+ "Routing Key"+CSV_SEPARATOR
														+ "Sequence No"+CSV_SEPARATOR
														+ "XML Payload"
														+ NEW_LINE_SEPARATOR;
	// constructor
	public BroadcastList(ControlBoard controlBoardFrame) {
		this.controlBoardFrame = controlBoardFrame;
	}
	
	// ***************************************************
	// *** Add a broadcast to the list 
	// ***************************************************
	public void AddBroadcast(String clientReceptionTime, String AMQPSendingTime, String type, String correlationId,
							String routingKey, String sequenceNo, String XMLPayload ) {
		String truncatedType = "";
		Broadcast broadcast = new Broadcast();
		
		broadcast.clientReceptionTime = clientReceptionTime;
		broadcast.AMQPSendingTime = AMQPSendingTime;
		
		
		truncatedType = type.substring(type.indexOf(BROADCAST_TYPE_PREFIX) + BROADCAST_TYPE_PREFIX.length());
		broadcast.correlationId = correlationId;
		
		broadcast.type = truncatedType;
		broadcast.routingKey = routingKey;
		broadcast.sequenceNo = sequenceNo;
		broadcast.XMLPayload = XMLPayload;
		
		broadcastList.add(broadcast);
		this.nbBroadcasts ++;
	}
	

	// ************************************************************************
	// *** Update the broadcast GUI table 
	// ************************************************************************
	public void updateGUITable(GUI_Panel broadcast_GUI_Panel) throws Exception {
		int broadcastRow = 0;
		int columnNb = 0;
		JTable table = broadcast_GUI_Panel.table;
		JTextField commentText = broadcast_GUI_Panel.commentText;
		DefaultTableModel dataModel = broadcast_GUI_Panel.dataModel;
		
		
	// update the GUI if the display is not paused (by hitting a key with the broadcast table selected ; see GUI_panel)	
	if (!broadcast_GUI_Panel.bPaused) {
		commentText.setText("Nb of broadcasts = " + this.nbBroadcasts);

	  for (Broadcast next : broadcastList) {
		  
		  // extend the table when reaching the end
		  if  (broadcastRow == table.getRowCount()) dataModel.addRow(new Object[] { broadcastRow });
	  
		  columnNb = 0; // {"#","Reception Time", "Sending Time","Type", "Routing Key",	  "Sequence No", "XML payload"};
		  table.setValueAt(broadcastRow+1,broadcastRow,columnNb); columnNb++ ;
		  table.setValueAt(next.clientReceptionTime,broadcastRow,columnNb); columnNb++;
		  table.setValueAt(next.AMQPSendingTime,broadcastRow,columnNb); columnNb++ ;
		  table.setValueAt(next.type,broadcastRow,columnNb); columnNb++ ;
		  table.setValueAt(next.correlationId,broadcastRow,columnNb); columnNb++ ;
		  table.setValueAt(next.routingKey,broadcastRow,columnNb); columnNb++ ;
		  table.setValueAt(next.sequenceNo,broadcastRow,columnNb); columnNb++ ;
		  table.setValueAt(next.XMLPayload,broadcastRow,columnNb); columnNb++ ;
		  broadcastRow++;
		  // extend the table 
		  //if  (broadcastRow == table.getRowCount()-1) dataModel.addRow(new Object[] { broadcastRow }); 
	  }
	 
	 // only sort once so that the user can change the sorting (sort by RK, etc.)
	 if (alreadySortedOnce == false) {
			table.getRowSorter().toggleSortOrder(1);
			table.getRowSorter().toggleSortOrder(1);
		 alreadySortedOnce = true;
	 }
	 
	}


	}
	
	// ***************************************************
	// *** WriteOrderBooksListCSVFile
	// ***************************************************	
	public void WriteCSVFile() {
		try {
	         File f = new File(FILE_BROADCAST_LIST_NAME);
	         if(!f.exists()) {
	            f.createNewFile();
	         }
	         
			FileWriter fileWriter = new FileWriter(FILE_BROADCAST_LIST_NAME);
			
			if(f.length() == 0) fileWriter.append(FILE_BROADCAST_LIST_HEADER);
				
			for (Broadcast next : broadcastList) {
				String row = 	next.clientReceptionTime + CSV_SEPARATOR
								+ next.AMQPSendingTime + CSV_SEPARATOR
								+ next.type + CSV_SEPARATOR
								+ next.correlationId + CSV_SEPARATOR
								+ next.routingKey + CSV_SEPARATOR
								+ next.sequenceNo + CSV_SEPARATOR
								+ next.XMLPayload
								+ NEW_LINE_SEPARATOR;
				fileWriter.append(row);
				}	

			
			fileWriter.flush();
			fileWriter.close();				
		
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}	
}
