import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Random;

import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

import org.apache.commons.lang.StringUtils;

public class ContractList {
	public LinkedList<Contract> contractList = new LinkedList<>();
	private SimpleDateFormat sdf = new SimpleDateFormat("E yyy/MM/dd HH:mm:ss SSS") ;
	public Log appLog = null;
	public int nbContracts = 0;
	private ControlBoard controlBoardFrame;
	
	private final static String FILE_CONTRACT_LIST_NAME = "Contract List.csv";
	private final static String NEW_LINE_SEPARATOR ="\n";
	private final static String CSV_SEPARATOR =";";
	private final static String FILE_CONTRACT_LIST_HEADER = "Contract ID"+CSV_SEPARATOR
														+ "Revision No"+CSV_SEPARATOR
														+ "Product"+CSV_SEPARATOR
														+ "Name"+CSV_SEPARATOR
														+ "Long Name"+CSV_SEPARATOR
														+ "Delivery Start"+ CSV_SEPARATOR
														+ "Delivery End"+ CSV_SEPARATOR
														+ "Predefined"+ CSV_SEPARATOR 
														+ "State" + CSV_SEPARATOR
														+ "Trading Phase" + CSV_SEPARATOR
														+ "Duration" + CSV_SEPARATOR
														+ "Activation Point" + CSV_SEPARATOR
														+ "Expiration Point" + CSV_SEPARATOR
														+ "Delivery Unit" 	+ NEW_LINE_SEPARATOR;

	// constructor
	public ContractList(Log appLog, ControlBoard controlBoardFrame) {
		this.appLog = appLog;
		this.controlBoardFrame = controlBoardFrame;
	}
	
	// ***************************************************
	// *** WriteCSVFileContractList
	// ***************************************************	
	public void WriteCSVFileContractList() {
		SimpleDateFormat dateFormatFile = new SimpleDateFormat("yyyyMMdd HH_mm_ss SSS") ;
		try {
			//FileWriter fileWriter = new FileWriter(dateFormatFile.format(new Date())+" "+FILE_CONTRACT_LIST_NAME);
			FileWriter fileWriter = new FileWriter(FILE_CONTRACT_LIST_NAME);
			
			for (Contract  next : contractList) {
				fileWriter.append(FILE_CONTRACT_LIST_HEADER);
				String row = 	next.getContractId() + CSV_SEPARATOR
								+ next.getRevisionNo() + CSV_SEPARATOR
								+ next.getProd() + CSV_SEPARATOR
								+ "'" + next.getName() + CSV_SEPARATOR // add a quote to avoid a conversion by Excel 
								+ next.getLongName() + CSV_SEPARATOR
								+ next.getDlvryStart() + CSV_SEPARATOR
								+ next.getDlvryEnd() + CSV_SEPARATOR
								+ next.getPredefined() + CSV_SEPARATOR
								+ next.getState() + CSV_SEPARATOR
								+ next.getTradingPhase() + CSV_SEPARATOR
								+ next.getDuration() + CSV_SEPARATOR
								+ next.getActPoint() + CSV_SEPARATOR
								+ next.getExpPoint() + CSV_SEPARATOR
								+ next.getDelUnits() + CSV_SEPARATOR
								+  NEW_LINE_SEPARATOR;
				fileWriter.append(row);
				
				// write all areas
				row = CSV_SEPARATOR+ next.getDeliveryAreaNb() + " Delivery areas"+ CSV_SEPARATOR  
						+ "Area"+ CSV_SEPARATOR
						+ "Area Name"+ CSV_SEPARATOR
						+ "Area Long Name"+ CSV_SEPARATOR
						+ "State"+ CSV_SEPARATOR
						+ "Trading Phase"+ CSV_SEPARATOR 
						+ "Delivery Start"+ CSV_SEPARATOR 
						+ "Delivery End"+ CSV_SEPARATOR 
						+ NEW_LINE_SEPARATOR;
				fileWriter.append(row);

				for (DeliveryArea  deliveryArea : next.getDeliveryAreaList()) {
					row = CSV_SEPARATOR+ CSV_SEPARATOR 
							+ deliveryArea.dlvryAreaId + CSV_SEPARATOR
							+ deliveryArea.dlvryAreaName + CSV_SEPARATOR
							+ deliveryArea.dlvryAreaLongName + CSV_SEPARATOR
							+ deliveryArea.state + CSV_SEPARATOR
							+ deliveryArea.tradingPhase + CSV_SEPARATOR
							+ deliveryArea.tradingPhaseStart + CSV_SEPARATOR
							+ deliveryArea.tradingPhaseEnd + CSV_SEPARATOR
							+ NEW_LINE_SEPARATOR;
					fileWriter.append(row);
				}
			}			
			fileWriter.flush();
			fileWriter.close();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = "the " + FILE_CONTRACT_LIST_NAME + " file cannot be accessed : please check it is well closed."; 
			JOptionPane.showMessageDialog(Globals.controlBoardFrame, msg);
		}
	}	
	
	// ***************************************************
	// *** toString
	// ***************************************************	
	@Override
	public String toString() {
		String s="";
		int contractNb = 0;
		for (Contract next : contractList) {
			contractNb++;
			s = "Contract #" + contractNb + ": "+ next.toString()+"\n";
		}
		return s; 
	}
	// ***************************************************
	// *** Display (writes in the GUI)
	// ***************************************************	
	public String Display(GUIWindow guiWindow) {
		String s="";
		int contractNb = 0;
		for (Contract next : contractList) {
			contractNb++;
			s = "Contract #" + contractNb + ": "+ next.toString()+"\n";
			guiWindow.AddText(s);
		}
		return s; 
	}
	
	// ***************************************************
	// *** AddContract
	// ***************************************************
	public int AddContract(Contract contract) {
		contractList.add(contract);
		this.nbContracts++;
		return contractList.size();
	}
	
	// ***************************************************
	// *** UpdateContract
	// ***************************************************
	public int UpdateContractList(Contract contract, OrderBooksList ordersBookList) {
		boolean contractIDFound = false;
		// replace the contract in the list if already exists
		for (Contract nextContract : contractList) {
			if (nextContract.getContractId().contentEquals(contract.getContractId())) {
				// the contract with the same contract ID has been found: 
				// update all contract attributes that can change + the delivery area list
				nextContract.setName(contract.getName());
				nextContract.setState(contract.getState());
				nextContract.setTradingPhase(contract.getTradingPhase());
				
				// replace the delivery area list for the current contract in contractList (nextContract)
				nextContract.getDeliveryAreaList().clear();
				nextContract.setDeliveryAreaList(contract.getDeliveryAreaList());
				
				// for each order books corresponding to the current Contract ID and area combination, 
				// update the obk "tradability" by updating it with the area trading state and phase : this automatically updates the obk "tradable" flag
				// note : not all areas change of state at the same time (check TRA100 products scedules doc in the API package)
				// this could be optimized by only updating when there is a change
				// changes take place at contract opening and closing mostly, but as well during the trading session for remote/XBID contract (when M7 gets disconnected / reconnected to XBID)
				for (DeliveryArea nextContractAreaList : contract.getDeliveryAreaList() ) {
					ordersBookList.refreshOrderBooksContractState(
								contract.getContractId(), 
								nextContractAreaList.dlvryAreaId ,
								nextContractAreaList.state, 
								nextContractAreaList.tradingPhase);
				}
				
				contractIDFound = true;
				String msg = "Contract : " + contract.getContractId() + " " + contract.getName() + " updated";
				Globals.guiWindowDebug.AddText(msg);
				appLog.logger.info(msg);
				break;
			}
		}
		// or just add it
		if (contractIDFound == false) {
			contractList.add(contract);			
		}

		return contractList.size();
	}
	
	// ******************************************************
	// *** IsXBIDContract : return "XBID", "LOCAL" or "NOT_FOUND"
	// ******************************************************
	public String IsXBIDContract(String ContractId) {
		String isXBIDContract = "NOT_FOUND";
		String product = "";
		
		for (Contract nextContract : contractList) {
			if (nextContract.getContractId().contentEquals(ContractId)) {
				product = nextContract.getProd();
				if (product.contains("XBID")) 	{ 
					isXBIDContract = "XBID";					
				}
				else {isXBIDContract = "LOCAL";}
				break;
			}
		}
		
		return isXBIDContract;
	}
	
	// ****************************************************************
	// *** ContractProduct : return the contract product or "NOT_FOUND"
	// ****************************************************************
	public String ContractProduct(String ContractId) {
		String product = "NOT_FOUND";
				
		for (Contract nextContract : contractList) {
			if (nextContract.getContractId().contentEquals(ContractId)) {
				product = nextContract.getProd();
				break;
			}
		}
		
		return product;
	}
	
	// ***************************************************
	// *** FindContractId based on the functional key product/delivery Start / delivery End
	// ***************************************************	
	public String FindContractId (String orderActionTime, String product, String deliveryStart, String deliveryEnd,GUIWindow guiWindow) {
		boolean contractFound = false;
		String contractId = "";
		LocalDateTime todayDateTime = LocalDateTime.now();
		LocalDate today = todayDateTime.toLocalDate();
		LocalDateTime tomorrowDateTime = todayDateTime.plusDays(1);
		LocalDate tomorrow = tomorrowDateTime.toLocalDate();
		
		String adaptedDeliveryStart = "";
		String adaptedDeliveryEnd = "";
		String orderActionDay = orderActionTime.substring(0, 10);
		String orderDeliveryStartDay = deliveryStart.substring(0,10);
		
		guiWindow.AddText("orderActionDay='"+ orderActionDay+"' orderDeliveryStartDay='"+ orderDeliveryStartDay+"'");
		
		// calculate whether the order was submitted for D or D+1
		// orderActionTime day = delivery Start day then D : search for a matching contract for delivery today
		// example: 2021-01-14T23:02:37.728Z vs 2021-01-15T22:00:00Z => D+1
		if (orderActionDay.contentEquals(orderDeliveryStartDay)) {
			// order was submitted for D
			// substitute the date with today's date
			guiWindow.AddText("contract for today");
			adaptedDeliveryStart = today + "T"+deliveryStart.substring(11,19)+ ".000Z";
			adaptedDeliveryEnd = today +"T"+ deliveryEnd.substring(11,19)+ ".000Z";
		}
		else { // order was submitted for D+1
			guiWindow.AddText("difference = " +StringUtils.difference(orderActionDay, orderDeliveryStartDay));

			guiWindow.AddText("contract for tomorrow");
			adaptedDeliveryStart = tomorrow + "T"+ deliveryStart.substring(11,19)+ ".000Z";
			adaptedDeliveryEnd = tomorrow + "T"+deliveryEnd.substring(11,19)+ ".000Z";
		}

		guiWindow.AddText("orderActionTime.substring(0, 10) ='"+ orderActionTime.substring(0, 10) + "' deliveryStart.substring(0,10)='"+ deliveryStart.substring(0,10) + "' => adaptedDeliveryStart="+ adaptedDeliveryStart + " adaptedDeliveryEnd="+adaptedDeliveryEnd);
		guiWindow.AddText("contractList size = " + String.valueOf(this.contractList.size()));

		int nbContract = 0;
		for (Contract next : this.contractList) {
			nbContract++;
			//guiWindow.AddText("--- Contract Name being examined #"+ nbContract + ":"+ next.getName());
			if (		next.getProd().contentEquals(product) 
					&& 	next.getDlvryStart().contentEquals(adaptedDeliveryStart) 
					&& 	next.getDlvryEnd().contentEquals(adaptedDeliveryEnd) ) {
				contractId = next.getContractId();
				contractFound = true;
				break;
			}
		}
		if (contractFound == false) {
			System.out.println(sdf.format(new Date()) + " contract not found for product=" + product + " deliveryStart=" + deliveryStart + " deliveryEnd="+deliveryEnd);
		}
		
		return contractId;
	}


// ***************************************************
// *** FindATradableContractAreaCouple
// ***************************************************
public String FindATradableContractAreaCouple() {
	String tradableContractID = "NOT_FOUND";
	String tradableArea = "NOT_FOUND";
	String tradableContractName = "NOT_FOUND"; 
	int maxContractNo = contractList.size();
	int randomContractNo = 0;
    int randomDANo = 0;
    
	while (tradableContractID == "NOT_FOUND") {
		
		// pick up a random contract No in the list
		Random randomContract = new Random();
	    randomContractNo = randomContract.nextInt(maxContractNo);

        int ContractItr = 1;
        boolean contractFound = false;
        
        // reach contract randomContractNo in the list
        for (Contract nextContract : contractList) {
        	if (ContractItr == randomContractNo) {
        		contractFound = true;
                // before checking the delivery areas ensure the contract "technical" state is ACTI
        		if (nextContract.getState().contentEquals("ACTI") && nextContract.getTradingPhase().contentEquals("CONT")) {
    				// try to find a tradable delivery area (in ACTI state) and the phase is a trading one (CONT or SDAT)
        			int maxDA = nextContract.getDeliveryAreaNb();
        			Random randDA = new Random();
        	        randomDANo = randDA.nextInt(maxDA);
        	        int areaItr = 1;
        	        boolean areaFound = false;
        	        // reach DA randomDANo in the DA List
        	        for (DeliveryArea nextArea : nextContract.getDeliveryAreaList() ) {
        	        	if (areaItr == randomDANo) {
        	        		areaFound = true;
        	        		// if the area is tradable, get contract ID and area
	        	        	if (nextArea.state.contentEquals("ACTI") 
	        	        				&& 
	        	        				(nextArea.tradingPhase.contentEquals("CONT")) || nextArea.tradingPhase.contentEquals("SDAT")) {
	        	        		tradableArea = nextArea.dlvryAreaId;
	        	        		tradableContractID = nextContract.getContractId();
	        	        		tradableContractName = nextContract.getName();
	        	        	}
        	        		break; // whatever is the DA state/phase exit the DA List
        	        	}
        	        	else areaItr++;
        	        }  // end of DA loop      	        
    			} // end of contract state/phase prerequisite
        		break; // whatever is the contract state, exit the contract loop since the contract has been found
        	}
        	else {
	        	ContractItr ++; // contract not reached yet 
	        	}
        }
	} // while
	return (tradableContractID+";"+tradableContractName+";"+tradableArea+ ";randomContractNo="+randomContractNo+ ";randomDANo ="+ randomDANo);
}


//*****************************************************************************
//*** FindATradableContractAreaForAProduct (random for a given product and area)
//*** search for a contract in an ACTIve state + phase CONT or SDAT
//******************************************************************************
public Contract FindATradableContractAreaForAProduct(String SearchForProductName, String SearchForDeliveryArea, GUIWindow guiWindow,Log appLog, boolean bDebug) {
	boolean bTradableContractIDFound = false;
	String msg = "";
	int trialsNb = 0;
	int MAX_TRIALS = 1000;
	int contractNbCurrentTrial=0;
	Contract foundContract = new Contract();
	
	Globals.guiWindowDebug.AddText("Find a tradable Contact start");
	appLog.logger.fine("Find a tradable Contact start");

	while (!bTradableContractIDFound && trialsNb < MAX_TRIALS) {
		trialsNb++;
		contractNbCurrentTrial = 0;
		
	   // select a random starting contract nb for the search
		Random randomContract = new Random();
	    int targetRandomContractNb = randomContract.nextInt(contractList.size());
	    msg = " targetRandomContractNb =" + targetRandomContractNb;
	    appLog.logger.fine(msg);
	    Globals.guiWindowDebug.AddText(msg);
	    
	     // reach contract randomContractNo in the list
		ListIterator<Contract> list_Iter = contractList.listIterator(); 

					  
		while (list_Iter.hasNext() && !bTradableContractIDFound) {
			Contract contract_Itr = list_Iter.next();
	    	 contractNbCurrentTrial ++;
	    	 // start searching for a tradable contract for the given area from a random place, to distribute the load
	    	 if (contractNbCurrentTrial >= targetRandomContractNb) {
		            // before checking the delivery areas ensure the contract "technical" state is ACTI and trade phase SDAT or CONT
		     	   String currentContractId  = contract_Itr.getContractId();
		     	   String currentContractName  = contract_Itr.getName();
		     	   String currentContractState = contract_Itr.getState();
		     	   String currentContractPhase = contract_Itr.getTradingPhase();
		     	   
		 			 // current contract not tradable: 
		 				msg = "Checking contract contractNbCurrentTrial " + contractNbCurrentTrial + " contract "  + currentContractId + " " + currentContractName + " "+ currentContractState + "/" + currentContractPhase;
		 				appLog.logger.fine(msg);
	     	    		
		     	   // ignore the product name : just use the area to increase the diversity
		     		//if (currentProductName.contentEquals(SearchForProductName) 
		     			if (currentContractState.contentEquals("ACTI") 
		     				&& currentContractPhase.contentEquals("CONT")) { // pre-select the contract only if tradable at "technical level": 
		 				// try to find a tradable delivery area (in ACTI state) and the phase is a trading one (CONT or SDAT)
			        
		     	        // find the DA passed as a parameter in the DA List
		     				// version 2 : fin
		     			Random randomDA = new Random();
		     			int targetRandomDANb = randomDA.nextInt(contract_Itr.getDeliveryAreaList().size());
		     			
		     			 msg = " current contract Id = " + currentContractId + " " + currentContractName + " targetRandomDANb =" + targetRandomDANb;
		     		    appLog.logger.fine(msg);
		     		    
		     			int currentDANb = 0;
		     			ListIterator<DeliveryArea> area_list_Iter = contract_Itr.getDeliveryAreaList().listIterator(); 
		     			boolean bTradableAreaFound = false;
		     			
		     			while (area_list_Iter.hasNext() && !bTradableAreaFound) { // while no tradable DA has been found
	     	        		DeliveryArea area_Itr = area_list_Iter.next();
		     				currentDANb ++;
		     	        	if (currentDANb >= targetRandomDANb) {

			     	        		String currentDA = area_Itr.dlvryAreaId;
			     	        		String currentDAState = area_Itr.state; 
			     	        		String currentDAPhase = area_Itr.tradingPhase;
			   	        		
			     	        		// if the searched area is found check if it is tradable, and if it is then get the contract ID
			     	        		// ignore the area
				        	        //if (currentDA.contentEquals(SearchForDeliveryArea) 
			     	        		// version 2: just find a random tradable DA
				        	        	if (currentDAState.contentEquals("ACTI") 
				        	        	&& 
				        	        	(currentDAPhase.contentEquals("CONT")) || currentDAPhase.contentEquals("SDAT")) 
				        	        	{
				        	        		bTradableContractIDFound = true;
				        	        		bTradableAreaFound = true;
				        	        		//tradableContractName = currentContractName;
				        	        		foundContract = contract_Itr;
				        	        		foundContract.setChosenAreaToTrade(currentDA);
			        	          		   msg = "for Order Generation (trader) : Contract FOUND : Product= "+ foundContract.getProd() 
			        	          		   		+ " ; Contract ID FOUND ="+ foundContract.getContractId() + ": '"+ foundContract.getName() 
			        	          		   		+ "' area state =" + currentDAState + " Area phase= " + currentDAPhase
			        	          		   		+ " area = " + foundContract.getChosenAreaToTrade() ;
			        	          		   //guiWindow.AddText(msg);
			        	          		   appLog.logger.fine(msg);
				        	        	}  
				        	      currentDANb++;
		     	        	}
		     	        }  // end of DA loop   
		 			} // end of contract product/state/phase prerequisite
		 			else {
		 			 // current contract not tradable: 
		 				msg = "NOT TRADABLE: contractNbCurrentTrial " + contractNbCurrentTrial + " contract "  + currentContractId + " " + currentContractName + " not tradable: " + currentContractState + "/" + currentContractPhase;
		 				appLog.logger.fine(msg);
		 			}
	    	 }
	    	}
	}
	Globals.guiWindowDebug.AddText(" exit 'Find a tradable contract' contractNbCurrentTrial = " + contractNbCurrentTrial);

	return (foundContract);
}

//***************************************************
//*** FindContractIdInfo returns functional contract info in an order Book object corresponding to a contract ID : product, contract delivery area state and phase
//***************************************************	

public OrderBook FindContractIdInfo (String contractId, String dlvryAreaId, GUIWindow guiWindow) {
	boolean contractFound = false;
	
	OrderBook  orderBook = new OrderBook();
	orderBook.setContractId(contractId);
	
	int nbContract = 0;

	// go through the contract list
	for (Contract nextContract : this.contractList) {
		nbContract++;
		// search for the requested contract Id
		if (nextContract.getContractId().contentEquals(contractId)) { // when the requested contract ID has been found in the contract list:
			
			orderBook.setContractID_Product(nextContract.getProd());
			orderBook.setContractID_Name(nextContract.getName());
			orderBook.setContractID_Product_DeliveryUnits(nextContract.getDelUnits());
			orderBook.setContractID_Product_Predefined(nextContract.getPredefined());
			contractFound = true;
	         // before checking the delivery areas ensure the contract "technical" state is ACTI, otherwise there is no area data (the contract is not tradable for any area)
	 		if (nextContract.getState().contentEquals("ACTI") && nextContract.getTradingPhase().contentEquals("CONT")) {

	 	        boolean areaFound = false;
	 			// search for the requested delivery area Id
	 	        for (DeliveryArea nextArea : nextContract.getDeliveryAreaList() ) {
	 	        	if (nextArea.dlvryAreaId.contentEquals(dlvryAreaId)) {
	 	        		
	 	        		areaFound = true;
	 	        		orderBook.setContractID_Area_State(nextArea.state);
	 	        		orderBook.setContractID_Area_Phase(nextArea.tradingPhase);
	 	        		
	 	        		// if the area is tradable, update the order book area tradable state 
	     	        	if (nextArea.state.contentEquals("ACTI")  
	     	        			&& (nextArea.tradingPhase.contentEquals("CONT")) || nextArea.tradingPhase.contentEquals("SDAT")) {
	     	        		orderBook.setContractID_Area_Is_Tradable(true);
	     	        	}
	     	        	else orderBook.setContractID_Area_Is_Tradable(false);
	     	        	
	 	        		break; //DA has been found : exit the DA List
	 	        	} // next area
	 	        } // end of area loop
	 	        
	 	        // if the contract was found but not the area, return the info
	 	        if (areaFound == false) {
	         		orderBook.setContractID_Area_State("AREA_NOT_FOUND");
	         		orderBook.setContractID_Area_Phase("AREA_NOT_FOUND");
	     			orderBook.setContractID_Area_Is_Tradable(false);    	        	
	 	        } 
	 		} // if the generic contract tradable
	 		else { // if generic contract NOT tradable
 	     		orderBook.setContractID_Area_State("N/A - Technical/Generic Contract not tradable");
 	     		orderBook.setContractID_Area_Phase("CLSD");
 	 			orderBook.setContractID_Area_Is_Tradable(false);
 	 	        } 
			break;
		} // search for the requested contract Id
	}	// go through the contract list
	
	if (contractFound == false) {
		System.out.println(sdf.format(new Date()) + " contract not found");
		orderBook.setContractID_Name("NOT_FOUND");
	}
	
	return orderBook;
}

//***************************************************
//*** FindContractIdInfo_Contract returns functional contract info in a CONTRACT object corresponding to a contract ID : product, contract delivery area state and phase
//***************************************************	

public Contract FindContractIdInfo_Contract (String contractId, GUIWindow guiWindow) {
	boolean contractFound = false;
	
	Contract contract= new Contract(); 
	contract.setContractId(contractId);	
	
	// go through the contract list
	for (Contract nextContract : this.contractList) {
		// search for the requested contract Id
		if (nextContract.getContractId().contentEquals(contractId)) { // when the requested contract ID has been found in the contract list:
			contractFound = true;

			contract.setActPoint(nextContract.getActPoint());
			contract.setDelUnits(nextContract.getDelUnits());
			contract.setDlvryEnd(nextContract.getDlvryEnd());
			contract.setDlvryStart(nextContract.getDlvryStart());
			contract.setDuration(nextContract.getDuration());
			contract.setExpPoint(nextContract.getExpPoint());
			contract.setLongName(nextContract.getLongName());
			contract.setPredefined(nextContract.getPredefined());
			contract.setRevisionNo(nextContract.getRevisionNo());
			contract.setState(nextContract.getState());
			contract.setTradingPhase(nextContract.getTradingPhase());
			contract.setProd(nextContract.getProd());
			contract.setName(nextContract.getName());

	} // search for the requested contract Id
	}	// go through the contract list
	
	if (contractFound == false) {
		System.out.println(sdf.format(new Date()) + " contract not found");
		contract.setName("CONTRACT_ID_NOT_FOUND");
	}
	
	return contract;
}

//***************************************************
//*** Find a closed contracts, for testing purposes
//***************************************************	

public Contract FindClosedContract () {
	boolean contractFound = false;
	
	Contract contract = new Contract(); 

	// go through the contract list
	for (Contract nextContract : this.contractList) {
		// search for the requested contract Id
		if (!nextContract.getState().contentEquals("ACTI")) { // when the requested contract ID has been found in the contract list:
			contractFound = true;

			contract.setContractId(nextContract.getContractId());
			contract.setActPoint(nextContract.getActPoint());
			contract.setDelUnits(nextContract.getDelUnits());
			contract.setDlvryEnd(nextContract.getDlvryEnd());
			contract.setDlvryStart(nextContract.getDlvryStart());
			contract.setDuration(nextContract.getDuration());
			contract.setExpPoint(nextContract.getExpPoint());
			contract.setLongName(nextContract.getLongName());
			contract.setPredefined(nextContract.getPredefined());
			contract.setRevisionNo(nextContract.getRevisionNo());
			contract.setState(nextContract.getState());
			contract.setTradingPhase(nextContract.getTradingPhase());
			contract.setProd(nextContract.getProd());
			contract.setName(nextContract.getName());

	} // search for the requested contract Id
	}	// go through the contract list
	
	if (contractFound == false) {
		System.out.println(sdf.format(new Date()) + " contract not found");
		contract.setName("CONTRACT_ID_NOT_FOUND");
	}
	
	return contract;
}

// ************************************
// pickUp_Tradable_Contract
// ************************************
public Contract pickUp_Tradable_Contract(ControlBoard controlBoardFrame) {
	String mainProduct = "";
	String backupProduct = "";
	String mainArea = "";
	String backupArea = "";
	String product = "";
	String area = "";
	String tradableContractId ="NOT_FOUND";
	Contract tradableContract = new Contract();
	
	String msg = "Start of pickUp_Tradable_Contract";
	appLog.logger.fine(msg);
	
	// pick up a random tradable contract and area combination
	Random randomContractSelection = new Random();
	int contractRandomNb= randomContractSelection.nextInt(3);
	
	if (contractRandomNb == 0) {
	 	mainProduct = "GB_Hour_Power";
	 	mainArea = "10YGB----------A";
	 	backupProduct = "GB_Half_Hour_Power";
	 	backupArea = mainArea ;	
	}
	else if (contractRandomNb == 1) {
		// local continental contract hourly
	 	mainProduct = "Intraday_Hour_Power";
	 	mainArea = "10YDE-RWENET---I";
	 	backupProduct = "XBID_Hour_Power"+ Globals.remoteProductSuffix;
	 	backupArea = mainArea;			
	}
	else  {
		 // XBID hourly
	 	mainProduct = "XBID_Hour_Power" + Globals.remoteProductSuffix;
	 	mainArea = "10YDE-RWENET---I";
	 	backupProduct = "Intraday_Hour_Power";			    	 	
	 	backupArea = mainArea;
	}		
	area = mainArea;
	product = mainProduct;

	// find a contract ID corresponding to the chosen product and area
	tradableContract =  this.FindATradableContractAreaForAProduct(product, area ,Globals.guiWindowDebug, appLog, false);
 	//controlBoardFrame.textAreaContracts.append(sdf.format(new Date()) + "ms: "+"tradableContractId=" + tradableContractId + "\n");
	if (tradableContract != null) {
		msg = "pickUp_Tradable_Contract: tradableContract = " + tradableContract.toString();
		appLog.logger.fine(msg);
		msg = "Tradable contract found: Product=" + tradableContract.getProd() + " Area=" + tradableContract.getChosenAreaToTrade()+ " contract Id = " + tradableContract.getContractId() + " : name =" + tradableContract.getName();
		Globals.guiWindowDebug.AddText(msg);
		//controlBoardFrame.textAreaContracts.append(sdf.format(new Date()) + msg);
		controlBoardFrame.contract_GUI_Panel.commentText.setText(msg);
		
		msg = "Level 3 pickUp_Tradable_Contract : tradableContract = " + tradableContract.toString();
		appLog.logger.fine(msg);
	}
	else {
		msg ="Tradable contract is null";
		appLog.logger.fine(msg);
	}

	// try the backup product and area if the main one did not enable to find a tradable contract
	/*if (tradableContractId.contains("NOT_FOUND")) {
		// switch to backup settings
		product = backupProduct;
		area = backupArea;
		
		tradableContractId = this.FindATradableContractAreaForAProduct(product, area ,Globals.guiWindowDebug, appLog, false);	
		//controlBoardFrame.textAreaContracts.append(sdf.format(new Date()) + "ms: "+" No tradable contract could not be found for " + mainProduct + " / " + mainArea + " : now trying with " + backupProduct + " / " + backupArea +  " : found ContractId="+tradableContractId+"\n");
	 	String msg = "tradableContractId=" + tradableContractId; 
	 	controlBoardFrame.contract_GUI_Panel.commentText.setText(msg);
	}*/

	// set contract attributes
	// contract Id
	/*tradableContract = this.FindContractIdInfo_Contract(tradableContractId, Globals.guiWindowDebug);
	tradableContract.setChosenAreaToTrade(area);*/
	

	return tradableContract;
}


// ************************************************************************
// *** Update the request_response GUI table 
// ************************************************************************
public void updateGUITable(GUI_Panel contract_GUI_Panel) {
	int row = 0;
	int columnNb = 0;
	JTable table = null;
	boolean alreadySortedOnce = false;
	
	table = contract_GUI_Panel.table;
	JTextField commentText = contract_GUI_Panel.commentText;
	DefaultTableModel dataModel = contract_GUI_Panel.dataModel;
	
	commentText.setText("Nb of contracts = " + this.nbContracts);

  for (Contract next : contractList) {

	  // extend the table when reaching the end
	  if  (row == table.getRowCount()) dataModel.addRow(new Object[] { row });
	  
	  columnNb = 0; // {"#" , "Contract ID","Revision No", "Product", "Name", "Long Name",
		//"Delivery Start", "Delivery End", "Predefined", "State", "Trading Phase", "Duration" ,"Activation Point", "Expiration Point", "Delivery Unit"};
	  table.setValueAt(String.format("%04d",row+1),row,columnNb); columnNb++ ;
	  table.setValueAt("CONTRACT",row,columnNb); columnNb++ ;
	  table.setValueAt(next.getContractId(),row,columnNb); columnNb++ ;
	  table.setValueAt(next.getRevisionNo(),row,columnNb); columnNb++;
	  table.setValueAt(next.getProd(),row,columnNb); columnNb++ ;
	  table.setValueAt(next.getName(),row,columnNb); columnNb++ ;
	  table.setValueAt(next.getLongName(),row,columnNb); columnNb++ ;
	  table.setValueAt(next.getDlvryStart(),row,columnNb); columnNb++ ;
	  table.setValueAt(next.getDlvryEnd(),row,columnNb); columnNb++ ;		  
	  table.setValueAt(next.getPredefined(),row,columnNb); columnNb++ ;
	  table.setValueAt(next.getState(),row,columnNb); columnNb++ ;
	  table.setValueAt(next.getTradingPhase(),row,columnNb); columnNb++ ;
	  table.setValueAt(next.getDuration(),row,columnNb); columnNb++ ;
	  table.setValueAt(next.getActPoint(),row,columnNb); columnNb++ ;
	  table.setValueAt(next.getExpPoint(),row,columnNb); columnNb++ ;
	  table.setValueAt(next.getDelUnits(),row,columnNb); columnNb++ ;
	  table.setValueAt(next.getDeliveryAreaNb(),row,columnNb); columnNb++ ;
	  table.setValueAt(next.getDeliveryAreaList(),row,columnNb); columnNb++ ;
	  
	  // display all delivery areas
	  if (next.getDeliveryAreaList().size() >0) {
		int areaColumnNb =0;
		int areaLabelColumnNb =0;
		int areaNb = 0;
		// display area column labels
		row++;   if  (row == table.getRowCount()) dataModel.addRow(new Object[] { row });
		
		table.setValueAt(String.format("%04d",row+1),row,areaLabelColumnNb); areaLabelColumnNb++ ;
		table.setValueAt("",row,areaLabelColumnNb); areaLabelColumnNb++ ;
		table.setValueAt("AREA",row,areaLabelColumnNb); areaLabelColumnNb++ ;
		table.setValueAt("#",row,areaLabelColumnNb); areaLabelColumnNb++ ;
		table.setValueAt("Area Id",row,areaLabelColumnNb); areaLabelColumnNb++ ;
		table.setValueAt("Name",row,areaLabelColumnNb); areaLabelColumnNb++ ;
		table.setValueAt("Long Name",row,areaLabelColumnNb); areaLabelColumnNb++ ;
		table.setValueAt("State",row,areaLabelColumnNb); areaLabelColumnNb++ ;
		table.setValueAt("Trading Phase",row,areaLabelColumnNb); areaLabelColumnNb++ ;
		table.setValueAt("Delivery Start",row,areaLabelColumnNb); areaLabelColumnNb++ ;
		table.setValueAt("Delivery End",row,areaLabelColumnNb); areaLabelColumnNb++ ;
		
		// display area list content
		
		for (DeliveryArea  deliveryArea : next.getDeliveryAreaList()) {
			areaNb++;
			areaColumnNb = 0;
			row++;  if  (row == table.getRowCount()) dataModel.addRow(new Object[] { row });
			table.setValueAt(String.format("%04d",row+1),row,areaColumnNb); areaColumnNb++ ;
			table.setValueAt("",row,areaColumnNb); areaColumnNb++ ;
			table.setValueAt("AREA",row,areaColumnNb); areaColumnNb++ ;
			table.setValueAt(String.format("%02d",areaNb),row,areaColumnNb); areaColumnNb++ ;
			table.setValueAt(deliveryArea.dlvryAreaId,row,areaColumnNb); areaColumnNb++ ;
			table.setValueAt(deliveryArea.dlvryAreaName,row,areaColumnNb); areaColumnNb++ ;
			table.setValueAt(deliveryArea.dlvryAreaLongName,row,areaColumnNb); areaColumnNb++ ;
			table.setValueAt(deliveryArea.state,row,areaColumnNb); areaColumnNb++ ;
			table.setValueAt(deliveryArea.tradingPhase,row,areaColumnNb); areaColumnNb++ ;
			table.setValueAt(deliveryArea.tradingPhaseStart,row,areaColumnNb); areaColumnNb++ ;
			table.setValueAt(deliveryArea.tradingPhaseEnd,row,areaColumnNb); areaColumnNb++ ;
		}
	  }	  
	  row++; // next contract
	  }
 
 // only sort once so that the user can change the sorting 
 if (alreadySortedOnce == false) {
	//	table.getRowSorter().toggleSortOrder(0);
//		table.getRowSorter().toggleSortOrder(0);
	 alreadySortedOnce = true;
 }

}

}
