import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

public class DeliveryAreaList {
	public List<DeliveryArea> deliveryAreaList = new ArrayList<DeliveryArea>();
	int nbAreas = 0;
	private ControlBoard controlBoardFrame;
	
	private SimpleDateFormat sdf = new SimpleDateFormat("E yyy/MM/dd HH:mm:ss SSS") ;
	
	private final static String FILE_DELIVERY_AREAS_LIST_NAME = "Delivery areas.csv";
	private final static String NEW_LINE_SEPARATOR ="\n";
	private final static String CSV_SEPARATOR =";";
	private final static String FILE_DELIVERY_AREAS_LIST_HEADER = 
														"dlvryAreaId"+CSV_SEPARATOR
														+ "Name"+CSV_SEPARATOR
														+ "Long name" + CSV_SEPARATOR
														+ "State"+CSV_SEPARATOR
														+ "remote State" + CSV_SEPARATOR
														+ "Market area Id"+CSV_SEPARATOR
														+ "revision No"
														+ NEW_LINE_SEPARATOR;
	// constructor
	public DeliveryAreaList(ControlBoard controlBoardFrame) {
		this.controlBoardFrame = controlBoardFrame;
	}
	// ***************************************************
	// *** WriteOrderBooksListCSVFile
	// ***************************************************	
	public void WriteCSVFile() throws Exception {
		try {
	         File f = new File(FILE_DELIVERY_AREAS_LIST_NAME);
	         if(!f.exists()) {
	            f.createNewFile();
	         }
	         
			FileWriter fileWriter = new FileWriter(FILE_DELIVERY_AREAS_LIST_NAME);
			
			if(f.length() == 0) fileWriter.append(FILE_DELIVERY_AREAS_LIST_HEADER);
				
			for (DeliveryArea next : deliveryAreaList) {
				String row = 	next.dlvryAreaId + CSV_SEPARATOR
								+ next.dlvryAreaName + CSV_SEPARATOR
								+ next.dlvryAreaLongName + CSV_SEPARATOR
								+ next.state + CSV_SEPARATOR
								+ next.remoteState + CSV_SEPARATOR
								+ next.mktAreaId + CSV_SEPARATOR
								+ next.revisionNo 
								+ NEW_LINE_SEPARATOR;
				fileWriter.append(row);
				}	
			
			fileWriter.flush();
			fileWriter.close();				
		
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = "the " + FILE_DELIVERY_AREAS_LIST_NAME + " file cannot be accessed : please check it is well closed."; 
			JOptionPane.showMessageDialog(controlBoardFrame, msg);
			
		}
	}
	
	// ***************************************************
	// *** toString
	// ***************************************************	
	@Override
	public String toString() {
		String s="";
		int areaNb = 0;
		for (DeliveryArea next : deliveryAreaList) {
			areaNb++;
			s = "\t Area #" + areaNb + ": "+ next.toString()+"\n";
		}
		return s; 
	}
	// ***************************************************
	// *** Display
	// ***************************************************	
	public String Display(GUIWindow guiWindow) {
		String s="";
		int areaNb = 0;
		for (DeliveryArea next : deliveryAreaList) {
			areaNb++;
			s = "Area #" + areaNb + ": "+ next.Display()+"\n";
			guiWindow.AddText(s);
		}
		return s; 
	}
	// ***************************************************
	// *** AddDeliveryArea
	// ***************************************************	
	public int AddDeliveryArea(DeliveryArea deliveryArea) {
		deliveryAreaList.add(deliveryArea);
		return deliveryAreaList.size();
	}
	
	// ***************************************************
	// *** UpdateDeliveryArea
	// ***************************************************	
	public int UpdateDeliveryArea(DeliveryArea deliveryArea) {
		boolean areaFound = false; 
		for (DeliveryArea next : deliveryAreaList) {
			// search the Area Id in the DA list (next) 
			if (next.dlvryAreaId.contentEquals(deliveryArea.dlvryAreaId) ) {
				// update the DA attributes if already in the list 
				next.state = deliveryArea.state;
				next.tradingPhase = deliveryArea.tradingPhase;
				next.tradingPhaseStart= deliveryArea.tradingPhaseStart;
				next.tradingPhaseEnd = deliveryArea.tradingPhaseEnd;
				
				areaFound = true;
				break;
			}
			areaFound = true; 
		}
		if (areaFound == false) {
			// add the DA if not already in the list (potential DA that opened later)
			System.out.println(sdf.format(new Date()) + " area not found, added to the list");
			deliveryAreaList.add(deliveryArea);
			
		}
		return deliveryAreaList.size();
	}
	
	// ***************************************************
	// *** GetAreaDetailsFromId
	// ***************************************************	
	public DeliveryArea GetAreaDetailsFromId(String dlvryAreaId) {
		DeliveryArea areaResult = new DeliveryArea();
		for (DeliveryArea next : deliveryAreaList) {
			if (next.dlvryAreaId.contentEquals(dlvryAreaId)) {
				areaResult.dlvryAreaLongName = next.dlvryAreaLongName;
				areaResult.dlvryAreaName = next.dlvryAreaName;
				areaResult.mktAreaId = next.mktAreaId;
				areaResult.state = next.state;
				areaResult.remoteState = next.remoteState;
				break;
			}
		}
		return areaResult;
	}
	
	// ***************************************************
	// *** GetRandomArea
	// ***************************************************	
	public DeliveryArea GetRandomArea() {
		DeliveryArea areaResult = new DeliveryArea();
		for (DeliveryArea next : deliveryAreaList) {
			areaResult.dlvryAreaId = next.dlvryAreaId;
				areaResult.dlvryAreaLongName = next.dlvryAreaLongName;
				areaResult.dlvryAreaName = next.dlvryAreaName;
				areaResult.mktAreaId = next.mktAreaId;
				areaResult.state = next.state;
				areaResult.remoteState = next.remoteState;
				break;
					}
		return areaResult;
	}
	
	// ************************************************************************
	// *** Update the GUI table
	// ************************************************************************
	public void updateGUITable(GUI_Panel deliveryAreas_GUI_Panel) {
		int areaRow = 0;
		int columnNb = 0;
		boolean alreadySortedOnce = false;
		
		JTable areasTable = deliveryAreas_GUI_Panel.table;
		JTextField commentText = deliveryAreas_GUI_Panel.commentText;
		DefaultTableModel dataModel = deliveryAreas_GUI_Panel.dataModel;
		
  
	    String comment = "Nb delivery : " +  deliveryAreaList.size();
	    
	    commentText.setText(comment);

		for (DeliveryArea next : deliveryAreaList) {

			// extend the table when reaching the end 
			if  (areaRow == areasTable.getRowCount()-1) dataModel.addRow(new Object[] { areaRow });
			
				// only update relevant values from the left to the right
				columnNb = 0;				
				// {"Contract Id", "Contract Name", "Area Id", "Nb of orders","Area phase", "Area State"};
				areasTable.setValueAt(areaRow+1,areaRow,columnNb); columnNb++ ;
				
				areasTable.setValueAt(next.dlvryAreaId,areaRow,columnNb); columnNb++ ;
				areasTable.setValueAt(next.dlvryAreaName,areaRow,columnNb); columnNb++ ;
				areasTable.setValueAt(next.dlvryAreaLongName,areaRow,columnNb); columnNb++ ;
				areasTable.setValueAt(next.state,areaRow,columnNb); columnNb++ ;
				areasTable.setValueAt(next.remoteState,areaRow,columnNb); columnNb++ ;
				areasTable.setValueAt(next.mktAreaId,areaRow,columnNb); columnNb++ ;


				areaRow++;

		}
		
		 if (alreadySortedOnce == false) {
			//	areasTable.getRowSorter().toggleSortOrder(8);
				//areasTable.getRowSorter().toggleSortOrder(8);
			alreadySortedOnce = true;
		 }

		//commentTextOrderBooks.setText(this.getNbEmptyTradableOrderBooks()+ " empty Order Book(s)");
	}

}
