import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

public class  OrderBooksList {

	public CopyOnWriteArrayList<OrderBook> orderBooksList = new CopyOnWriteArrayList<>();
	public int nbOrderBooks = 0;
	public int nbTradableOrderBooks = 0;
	public int nbEmptyTradableOrderBooks = 0;
	public DeliveryAreaList deliveryAreaList;
	public ContractList contractList;
	private SimpleDateFormat sdf = new SimpleDateFormat("yyy/MM/dd HH:mm:ss.sss") ;
		
	// constructor
	public OrderBooksList(DeliveryAreaList deliveryAreaList, ContractList contractList) {
		this.deliveryAreaList = deliveryAreaList;
		this.contractList = contractList;
	}
	
	public int getNbTradableOrderBooks() {
		return nbTradableOrderBooks;
	}
	public void setNbTradableOrderBooks(int nbTradableOrderBooks) {
		this.nbTradableOrderBooks = nbTradableOrderBooks;
	}
	public int getNbEmptyTradableOrderBooks() {
		return nbEmptyTradableOrderBooks;
	}
	public void setNbEmptyTradableOrderBooks(int nbEmptyTradableOrderBooks) {
		this.nbEmptyTradableOrderBooks = nbEmptyTradableOrderBooks;
	}
	
	public List<OrderBook> getOrderBooksList() {
		return orderBooksList;
	}
	public void setOrderBooksList(CopyOnWriteArrayList<OrderBook> orderBooksList) {
		this.orderBooksList = orderBooksList;
	}
	public int getNbOrderBooks() {
		return nbOrderBooks;
	}
	public void setNbOrderBooks(int nbOrderBooks) {
		this.nbOrderBooks = nbOrderBooks;
	}
	
	private final static String FILE_ORDER_BOOKS_LIST_NAME = "Order Books List.csv";
	private final static String NEW_LINE_SEPARATOR ="\n";
	private final static String CSV_SEPARATOR =";";
	private final static String FILE_ORDER_BOOKS_LIST_HEADER = 
														"Last Update Time"+CSV_SEPARATOR
														+ "Contract Id"+CSV_SEPARATOR 
														+ "Contract Name"+CSV_SEPARATOR
														+ "Delivery Area Id"+CSV_SEPARATOR
														
														+ "Best Bid"+CSV_SEPARATOR
														+ "Best Ask"+CSV_SEPARATOR
														+ "Last Price"+CSV_SEPARATOR
														+ "Last Qty"+CSV_SEPARATOR
														
														+ "Product"+CSV_SEPARATOR
														+ "Predefined"+CSV_SEPARATOR
														+ "Del Unit"+CSV_SEPARATOR

														+ "Revision No"+CSV_SEPARATOR
														+ "Nb of orders"+CSV_SEPARATOR
														+ "Contract is Tradable"+CSV_SEPARATOR
														+ "Area Phase"+CSV_SEPARATOR
														+ "Area State"
														+ NEW_LINE_SEPARATOR;
	
	private String contractID_Product = "";
	private String contractID_Area_State = "";
	private String contractID_Area_Phase = "";
	private boolean contractID_Area_Is_Tradable = false;
	
	// ***************************************************
	// *** WriteOrderBooksListCSVFile
	// ***************************************************	
	public void WriteOrderBooksListCSVFile() {
		try {
	         File f = new File(FILE_ORDER_BOOKS_LIST_NAME);
	         if(!f.exists()) {
	            f.createNewFile();
	         }
	         
			FileWriter fileWriter = new FileWriter(FILE_ORDER_BOOKS_LIST_NAME);
			
			if(f.length() == 0) fileWriter.append(FILE_ORDER_BOOKS_LIST_HEADER);
				
		ListIterator<OrderBook> list_Iter = orderBooksList.listIterator(); 
		while (list_Iter.hasNext()) {
			OrderBook orderBookItr = list_Iter.next();
			
			// //"Best Bid", "Best Ask" 
			String bestBid;
			String bestAsk;
			
			if ("".contentEquals( orderBookItr.getsBestBid() )) {
				bestBid = "-";
			} 	
			else bestBid = String.format("%.2f",orderBookItr.getfBestBid());
						
			if ("".contentEquals( orderBookItr.getsBestAsk() )) {
				bestAsk = "-";
			} 	
			else bestAsk = String.format("%.2f",orderBookItr.getfBestAsk());
				
				String row = 	orderBookItr.getLastUpdateTime() + CSV_SEPARATOR
								+ orderBookItr.getContractId() + CSV_SEPARATOR
								+ orderBookItr.getContractID_Name() + CSV_SEPARATOR
								+ orderBookItr.getDlvryAreaId() + CSV_SEPARATOR
								+ bestBid + CSV_SEPARATOR
								+ bestAsk + CSV_SEPARATOR
								+ orderBookItr.getLastQty() + CSV_SEPARATOR
								+ orderBookItr.getLastPx() + CSV_SEPARATOR
								+ orderBookItr.getContractID_Product() + CSV_SEPARATOR
								+ orderBookItr.getContractID_Product_Predefined() + CSV_SEPARATOR
								+ orderBookItr.getContractID_Product_DeliveryUnits() + CSV_SEPARATOR
								+ orderBookItr.getRevisionNo() + CSV_SEPARATOR
								+ orderBookItr.getNbActiveOrders() + CSV_SEPARATOR
								+ orderBookItr.getContractID_Area_Is_Tradable()+ CSV_SEPARATOR
								+ orderBookItr.getContractID_Area_Phase() + CSV_SEPARATOR
								+ orderBookItr.getContractID_Area_State()
								+ NEW_LINE_SEPARATOR;
								
				fileWriter.append(row);
				}	
			
			fileWriter.flush();
			fileWriter.close();				
		
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = "the " + FILE_ORDER_BOOKS_LIST_NAME + " file cannot be accessed : please check it is well closed."; 
			JOptionPane.showMessageDialog(Globals.controlBoardFrame, msg);
		}
	}		
	
	// ***************************************************
	// *** Add an order book to the list 
	// ***************************************************
	public int AddOrderBook(String contractId, String dlvryAreaId, String revisionNo, 
							float bestBid, float bestAsk,
							float lastPx, float lastQty, float totalQty, String lastTradeTime, String lowPx, String highPx, 
							int spread,int depthAsk_NbOrders, int depthBid_NbOrders, int nbOrders, ContractList contractList,
							 Log appLog) {
		OrderBook  orderBook = new OrderBook();
		
		orderBook.setContractId(contractId);
		orderBook.setDlvryAreaId(dlvryAreaId);
		orderBook.setRevisionNo(revisionNo);
		
		orderBook.setfBestBid(bestBid);
		orderBook.setfBestAsk(bestAsk);
		
		orderBook.setHighPx(highPx);
		orderBook.setLastPx(lastPx);
		orderBook.setLastQty(lastQty);
		orderBook.setTotalQty(totalQty);
		orderBook.setLastTradeTime(lastTradeTime);
		orderBook.setLowPx(lowPx);
		orderBook.setHighPx(highPx);
		
		orderBook.setSpread(spread);
		orderBook.setDepthAsk_NbOrders(depthAsk_NbOrders);
		orderBook.setDepthBid_NbOrders(depthBid_NbOrders);
		orderBook.setNbOrders(nbOrders); 
	
		orderBook.setLastUpdateTime(sdf.format(new Date()));
		
		// find contract related info
	/*	Contract contractInfo = new Contract();
		contractInfo = contractList.FindContractIdInfo_Contract(contractId, Globals.guiWindowDebug);
		orderBook.setContractID_Product(contractInfo.getProd());
		orderBook.setContractID_Product_Predefined(contractInfo.getPredefined()());
		orderBook.setContractID_Product_DeliveryUnits(contractInfo.getDelUnits());
		orderBook.setContractID_Name(contractInfo.getContractID_Area_Phase());
		orderBook.setContractID_Area_Phase(contractInfo.get);
		orderBook.setContractID_Area_State(contractInfo.getContractID_Area_State());
		orderBook.setContractID_Area_Is_Tradable(contractInfo.getContractID_Area_Is_Tradable());	*/	
		
		OrderBook  orderBookTemp = new OrderBook();
		orderBookTemp = contractList.FindContractIdInfo (contractId, dlvryAreaId, null);
		
		orderBook.setContractID_Product(orderBookTemp.getContractID_Product());
		orderBook.setContractID_Product_Predefined(orderBookTemp.getContractID_Product_Predefined());
		orderBook.setContractID_Product_DeliveryUnits(orderBookTemp.getContractID_Product_DeliveryUnits());
		orderBook.setContractID_Name(orderBookTemp.getContractID_Name());
		orderBook.setContractID_Area_Phase(orderBookTemp.getContractID_Area_Phase());
		orderBook.setContractID_Area_State(orderBookTemp.getContractID_Area_State());
		orderBook.setContractID_Area_Is_Tradable(orderBookTemp.getContractID_Area_Is_Tradable());
		
		// find area info
		DeliveryArea areaDetails = new DeliveryArea();
		areaDetails = deliveryAreaList.GetAreaDetailsFromId(dlvryAreaId);
		orderBook.setDlvryAreaName(areaDetails.dlvryAreaName);
		orderBook.setDlvryAreaLongName(areaDetails.dlvryAreaLongName);
		
		// update stats at order book list level
		if (orderBook.getContractID_Area_Is_Tradable()) {
			this.nbTradableOrderBooks ++;
			if (nbOrders == 0) this.nbEmptyTradableOrderBooks ++;			
		}
		
		orderBooksList.add(orderBook);
		 if (!"10YLV-1001A00074".contentEquals(orderBook.getDlvryAreaId())) this.setNbOrderBooks(this.getNbOrderBooks() + 1);
		
		return this.getNbOrderBooks();
	}
	
	// ************************************************************************
	// *** Update an order book in the list 
	// ************************************************************************
	public void UpdateOrderBooksList(String contractId, String dlvryAreaId, String revisionNo, 
			float bestBid, float bestAsk,
			float lastPx, float lastQty, float totalQty, String lastTradeTime, String lowPx, String highPx, 
			int spread,int depthAsk_NbOrders, int depthBid_NbOrders, int deltaNbOrders, 
			ContractList contractList,
			 Log appLog) throws Exception {
		
	boolean bExecute = true;
		
	if (bExecute) {
		boolean Obk_Found = false;
		//System.out.println("UpdateOrderBooksList START");
		
		ListIterator<OrderBook> list_Iter = orderBooksList.listIterator(); 
		while (list_Iter.hasNext()) {
			OrderBook orderBookItr = list_Iter.next();
			
			if ( orderBookItr.getContractId().contentEquals(contractId) && orderBookItr.getDlvryAreaId().contentEquals(dlvryAreaId)) {
				// BG already listed : just update values
				Obk_Found = true;
	
				orderBookItr.setRevisionNo(revisionNo);

				orderBookItr.setfBestBid(bestBid);
				orderBookItr.setfBestAsk(bestAsk);
				
				orderBookItr.setHighPx(highPx);
				orderBookItr.setLastPx(lastPx);
				orderBookItr.setLastQty(lastQty);
				orderBookItr.setTotalQty(totalQty);
				orderBookItr.setLastTradeTime(lastTradeTime);
				orderBookItr.setLowPx(lowPx);
				orderBookItr.setHighPx(highPx);

				orderBookItr.setSpread(spread);
				orderBookItr.setDepthAsk_NbOrders(depthAsk_NbOrders);
				orderBookItr.setDepthBid_NbOrders(depthBid_NbOrders);
				
				//nb of active orders: take into account orders exiting the obk (qty =0), because hibernated, deleted or fully traded (or out of order book depth for XBID obks)
				int previousNbActiveOrders = orderBookItr.getNbActiveOrders() ;
				int newNbActiveOrders = orderBookItr.getNbActiveOrders() + deltaNbOrders;
				if (newNbActiveOrders<0) newNbActiveOrders = 0; // potential bug protection
				orderBookItr.setNbActiveOrders(newNbActiveOrders);
				
				orderBookItr.setLastUpdateTime(sdf.format(new Date()));
				
				boolean bPreviousObkTradableState = orderBookItr.getContractID_Area_Is_Tradable();
				
				// find contract related info
				OrderBook  orderBookTemp = new OrderBook(); // using an order book just to pass contract info to an easy object
				orderBookTemp = contractList.FindContractIdInfo (contractId, dlvryAreaId, null);
				
				// orderBookItr.setContractID_Product(orderBookTemp.getContractID_Product()); // product does not change
				orderBookItr.setContractID_Name(orderBookTemp.getContractID_Name());
				orderBookItr.setContractID_Area_Phase(orderBookTemp.getContractID_Area_Phase());
				orderBookItr.setContractID_Area_State(orderBookTemp.getContractID_Area_State());
				orderBookItr.setContractID_Area_Is_Tradable(orderBookTemp.getContractID_Area_Is_Tradable());
				boolean bNewObkTradableState = orderBookTemp.getContractID_Area_Is_Tradable();

				String msg ="order book update: "+ orderBookItr.getContractID_Name()
				+ " bPreviousObkTradableState = " + bPreviousObkTradableState
				+ " bNewObkTradableState = "+ bNewObkTradableState
				+ "  nbTradableOrderBooks = " + nbTradableOrderBooks
				+ "  nbEmptyTradableOrderBooks = " + nbEmptyTradableOrderBooks;
				
				appLog.logger.info(msg);
				System.out.println(msg);
				Globals.guiWindowDebug.AddText(msg);
				
				if (bPreviousObkTradableState != bNewObkTradableState) {
					if (bNewObkTradableState)  { // the order book has become tradable
						this.nbTradableOrderBooks ++;
						appLog.logger.info("order book update: "+ orderBookItr.getContractID_Name() + " becomes tradable: nbTradableOrderBooks ++ = " + nbTradableOrderBooks);
					}
					else { // the order book is not tradable anymore
						this.nbTradableOrderBooks --;
						appLog.logger.info("order book update: "+ orderBookItr.getContractID_Name() + " NOT tradable anymore: nbTradableOrderBooks -- = " + nbTradableOrderBooks);
					}
				}
				if (!"10YLV-1001A00074".contentEquals(orderBookItr.getDlvryAreaId())) {
					if ( bNewObkTradableState && (previousNbActiveOrders != newNbActiveOrders)) { // if the obk is tradable and nb active orders decreased to zero
						if (previousNbActiveOrders > 0 && newNbActiveOrders == 0) {
							this.nbEmptyTradableOrderBooks ++; // temp issue with ASIM config
							appLog.logger.info("order book update: "+ orderBookItr.getContractID_Name() + " newNbActiveOrders down to zero = " + newNbActiveOrders + " nbEmptyTradableOrderBooks ++ ="+ nbEmptyTradableOrderBooks);
						}
					}
						
					if ( bNewObkTradableState && (previousNbActiveOrders != newNbActiveOrders)) { // if the obk is tradable and nb active orders decreased to zero
							if (previousNbActiveOrders ==0  && newNbActiveOrders > 0) {
								this.nbEmptyTradableOrderBooks --;
								appLog.logger.info("order book update: "+ orderBookItr.getContractID_Name() + " newNbActiveOrders increases from zero = " + newNbActiveOrders + " nbEmptyTradableOrderBooks -- ="+ nbEmptyTradableOrderBooks);
							}
					}
				}
			
				break;
			}
		}
		// if Obk could not be found in the list : add it
		if (Obk_Found == false && (!"10YLV-1001A00074".contentEquals(dlvryAreaId))) {
		
			this.AddOrderBook(contractId, dlvryAreaId, revisionNo, bestBid, bestAsk,
					lastPx, lastQty, totalQty, lastTradeTime, lowPx, highPx, 
					spread,depthAsk_NbOrders, depthBid_NbOrders, deltaNbOrders, contractList,appLog);
		}
		
		int nbEmptyTradableObk = this.countTradableEmptyOrderBooks();
		this.setNbEmptyTradableOrderBooks(nbEmptyTradableObk);
		this.setNbTradableOrderBooks(this.countTradableOrderBooks());
		
		appLog.logger.finer("inside update obk : Obk_Found = " + Obk_Found + " NB empty tradable obk = " + nbEmptyTradableObk);

	}
	}
	
	// ************************************************************************
	// *** Count how many tradable order books do not have any order
	// ************************************************************************
	public int countTradableEmptyOrderBooks() {
		int nbEmptyTradableOrderBooks = 0;
		
		ListIterator<OrderBook> list_Iter = orderBooksList.listIterator(); 
		while (list_Iter.hasNext()) {
			OrderBook orderBookItr = list_Iter.next();
			
			if (!"10YLV-1001A00074".contentEquals(orderBookItr.getDlvryAreaId())  && orderBookItr.getContractID_Area_Is_Tradable() && orderBookItr.getNbActiveOrders() == 0) {
				nbEmptyTradableOrderBooks ++;
			}
		}
		this.nbEmptyTradableOrderBooks = nbEmptyTradableOrderBooks;
		return nbEmptyTradableOrderBooks;		
	}
	// ************************************************************************
	// *** Count how many tradable order books 
	// ************************************************************************
	public int countTradableOrderBooks() {
		int nbTrdableOrderBooks = 0;
		ListIterator<OrderBook> list_Iter = orderBooksList.listIterator(); 
		while (list_Iter.hasNext()) {
			OrderBook orderBookItr = list_Iter.next();
			if (!"10YLV-1001A00074".contentEquals(orderBookItr.getDlvryAreaId())  && orderBookItr.getContractID_Area_Is_Tradable() ) {
				nbTrdableOrderBooks ++;
			}
		}
		this.nbTradableOrderBooks = nbTrdableOrderBooks;
		return nbTrdableOrderBooks;		
	}
	
	// ************************************************************************
	// *** Refresh order book tradable state 
	// ************************************************************************
	public void refreshOrderBooksContractState(String contractId, String dlvryAreaId, String state, String phase) {
		boolean orderBook_Found = false;
	
		ListIterator<OrderBook> list_Iter = orderBooksList.listIterator(); 
		while (list_Iter.hasNext()) {
			OrderBook orderBookItr = list_Iter.next();
			
			// find the order book for the couple contractId/areaId
			if (orderBookItr.getContractId().contentEquals(contractId) && orderBookItr.getDlvryAreaId().contentEquals(dlvryAreaId) ) {
				// update the state, phase 
				orderBook_Found = true;
				orderBookItr.setContractID_Area_State(state);
				orderBookItr.setContractID_Area_Phase(phase);
				break;
			}
		}
	// add msg in Log if obk not found
	// to be completed
	}
	
	// ************************************************************************
	// *** Return best Bid of a given order book
	// ************************************************************************
	public OrderBook FindObkDetails(String contractId, String dlvryAreaId) {
		boolean orderBook_Found = false;
		float f_BestBid = 0;
		
		OrderBook orderBook = new OrderBook();
	
		ListIterator<OrderBook> list_Iter = orderBooksList.listIterator(); 
		while (list_Iter.hasNext()) {
			OrderBook orderBookItr = list_Iter.next();
			
			// find the order book for the couple contractId/areaId
			if (orderBookItr.getContractId().contentEquals(contractId) && orderBookItr.getDlvryAreaId().contentEquals(dlvryAreaId) ) {
				// update the state, phase 
				orderBook_Found = true;
				orderBook = orderBookItr;
				break;
			}
		}
	// add msg in Log if obk not found
	return orderBook;
	}

	// ************************************************************************
	// *** Update the GUI table displaying Order Books data 
	// ************************************************************************
//	public void updateGUITable(GUI_Panel orderBooks_GUI_Panel) throws Exception {
//		int orderBookRow = 0;
//		int columnNb = 0;
//		boolean alreadySortedOnce = false;
//		boolean bypass = true;
//		
//		JTable orderBooksTable = orderBooks_GUI_Panel.table;
//		JTextField commentTextOrderBooks = orderBooks_GUI_Panel.commentText;
//		DefaultTableModel dataModel = orderBooks_GUI_Panel.dataModel;
//		
//		Runnable r = new ThreadUpdateOrderBooksGUI(this , orderBooks_GUI_Panel, appLog);
//		Thread t1 = new Thread(r);
//		t1.start();
//		
//		if (!bypass) {
//	    int nbTradableObk = this.nbTradableOrderBooks;
//	    int nbEmptyTradableObk = this.nbEmptyTradableOrderBooks;
//	    float ratioEmptyObk = ((float)nbEmptyTradableObk/(float)nbTradableObk)*100;
//	    
//	    int nbTradableObk2 = this.countTradableOrderBooks();
//	    int nbEmptyTradableObk2 = this.countTradableEmptyOrderBooks();
//	    float ratioEmptyObk2 ;
//	    if (nbTradableObk2>0) ratioEmptyObk2 = ((float)nbEmptyTradableObk2/(float)nbTradableObk2)*100;
//	    else ratioEmptyObk2 = 0;
//	    
//	    String comment = "Nb empty order books = "+ nbEmptyTradableObk2 + " / " + nbTradableObk2 + " (" + String.format("%.0f%%",ratioEmptyObk2)+ ")";
//	    
//		commentTextOrderBooks.setText(comment);
//
//		ListIterator<OrderBook> list_Iter = orderBooksList.listIterator(); 
//		while (list_Iter.hasNext()) {
//			OrderBook orderBookItr = list_Iter.next();
//
//			// extend the table when reaching the end 
//			if  (orderBookRow == orderBooksTable.getRowCount()-1) dataModel.addRow(new Object[] { orderBookRow });
//			
//			// only process tradable order books without any active order 
//			//if (!orderBookItr.getContractID_Name().contentEquals("NOT_FOUND") && orderBookItr.getContractID_Area_Is_Tradable() && orderBookItr.getNbActiveOrders()==0) {
//			//if (orderBookItr.getContractID_Area_Is_Tradable() && orderBookItr.getNbActiveOrders()==0) {
//			if (orderBookItr.getContractID_Area_Is_Tradable()) {
//				// only update relevant values from the left to the right
//				columnNb = 0;				
//				orderBooksTable.setValueAt(orderBookRow+1,orderBookRow,columnNb); columnNb++ ;
//				
//				orderBooksTable.setValueAt(orderBookItr.getContractId(),orderBookRow,columnNb); columnNb++ ;
//				orderBooksTable.setValueAt(orderBookItr.getDlvryAreaId(),orderBookRow,columnNb); columnNb++ ;
//				
//				orderBooksTable.setValueAt(orderBookItr.getContractID_Product(),orderBookRow,columnNb); columnNb++ ;
//				orderBooksTable.setValueAt(orderBookItr.getContractID_Name(),orderBookRow,columnNb); columnNb++ ;
//				orderBooksTable.setValueAt(orderBookItr.getDlvryAreaName(),orderBookRow,columnNb); columnNb++ ;
//				
//				// //"Best Bid", "Best Ask" 
//				String bestBid;
//				String bestAsk;
//				
//				if ("".contentEquals( orderBookItr.getsBestBid() )) {
//					bestBid = "-";
//				} 	
//				else bestBid = String.format("%.2f",orderBookItr.getfBestBid());
//				
//				orderBooksTable.setValueAt(bestBid,orderBookRow,columnNb); columnNb++ ;
//				
//				if ("".contentEquals( orderBookItr.getsBestAsk() )) {
//					bestAsk = "-";
//				} 	
//				else bestAsk = String.format("%.2f",orderBookItr.getfBestAsk());
//				
//				orderBooksTable.setValueAt(bestAsk,orderBookRow,columnNb); columnNb++ ;
//		
//				//"Last qty","Last price",  "last update", "last trade", "Nb of orders",
//				
//				orderBooksTable.setValueAt(String.format("%.2f",orderBookItr.getLastQty()),orderBookRow,columnNb); columnNb++ ;
//				orderBooksTable.setValueAt(String.format("%.2f",orderBookItr.getLastPx()),orderBookRow,columnNb); columnNb++ ;
//
//				orderBooksTable.setValueAt(orderBookItr.getLastUpdateTime(),orderBookRow,columnNb); columnNb++ ;
//				orderBooksTable.setValueAt(orderBookItr.getLastTradeTime(),orderBookRow,columnNb); columnNb++ ;
//				orderBooksTable.setValueAt(String.format("%03d", orderBookItr.getNbActiveOrders()),orderBookRow,columnNb); columnNb++ ;
//				
//				// "Tradable","Area phase", "Area State"};
//				orderBooksTable.setValueAt(orderBookItr.getContractID_Area_Is_Tradable(),orderBookRow,columnNb); columnNb++ ;
//				orderBooksTable.setValueAt(orderBookItr.getContractID_Area_Phase(),orderBookRow,columnNb); columnNb++ ;
//				orderBooksTable.setValueAt(orderBookItr.getContractID_Area_State(),orderBookRow,columnNb); columnNb++ ;
//				orderBookRow++;
//			}
//		}
//		
//		 if (alreadySortedOnce == false) {
//				orderBooksTable.getRowSorter().toggleSortOrder(8);
//				orderBooksTable.getRowSorter().toggleSortOrder(8);
//				alreadySortedOnce = true;
//		 }
//	}
//}
	
}
