import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

public class RoutingKeysList {

	public CopyOnWriteArrayList<RoutingKey> routingKeysList = new CopyOnWriteArrayList<>();
	private int nbGaps = 0;
	
	public int getNbGaps() {
		return nbGaps;
	}

	private SimpleDateFormat sdf = new SimpleDateFormat("E yyy/MM/dd HH:mm:ss SSS") ;
	private final static String FILE_RK_LIST_NAME = "RK List.csv";
	private final static String FILE_MISSED_RK_LIST_NAME = "Missed RK List.csv";
	private final static String FILE_RK_GAP_EVENTS_NAME = "RK GAP Events.csv";
	private final static String FILE_RK_SEQNO_EVENTS_NAME = "RK Seq No Events.csv";
	private final static String NEW_LINE_SEPARATOR ="\n";
	private final static String CSV_SEPARATOR =";";
	private final static String  TABULATION = "\t";
	private final static String FILE_RK_LIST_HEADER = 	"Routing Key"+CSV_SEPARATOR
														+ "Gap detected"+CSV_SEPARATOR
														+ "Nb of received broadasts"+CSV_SEPARATOR
														+ "Previous Seq No"+CSV_SEPARATOR
														+ "Last Sequence No"+ CSV_SEPARATOR
														+ "Increment"+ CSV_SEPARATOR
														+ "Missed Seq No"+ CSV_SEPARATOR 
														+ "Previous Msg Type" + CSV_SEPARATOR
														+ "Last Msg Type"+NEW_LINE_SEPARATOR;
	private final static String FILE_MISSED_RK_LIST_HEADER = 	"Routing Key"+CSV_SEPARATOR
			+ "Missed seqNo"+CSV_SEPARATOR
			+ "Received?"+CSV_SEPARATOR
			+ "Missing time"+CSV_SEPARATOR
			+ "Reception time"+CSV_SEPARATOR
			+ "Time Difference ms"
			+ NEW_LINE_SEPARATOR;
	
	// ***************************************************
	// *** WriteCSVFile
	// ***************************************************	
	public void WriteFileRKList(String fileName) {
		try {
			FileWriter fileWriter = new FileWriter(fileName);
			fileWriter.append(FILE_RK_LIST_HEADER);
			
			for (RoutingKey next : routingKeysList) {
				String row = 	next.getRoutingKeyName() + CSV_SEPARATOR
								+ next.GetGapDetected() + CSV_SEPARATOR
								+ next.getNbOfBroadcasts() + CSV_SEPARATOR 
								+ next.getPreviousSequenceNo() + CSV_SEPARATOR
								+ next.getCurrentSequenceNo() + CSV_SEPARATOR
								+ (next.getCurrentSequenceNo()-next.getPreviousSequenceNo()) + CSV_SEPARATOR
								+ next.getMissingSequenceNo() + CSV_SEPARATOR
								+ next.getPreviousBroadcastMsgType() + CSV_SEPARATOR
								+ next.getCurrentBroadcastMsgType() + NEW_LINE_SEPARATOR;
				fileWriter.append(row);
			}			
			fileWriter.flush();
			fileWriter.close();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}
	
	// ***************************************************
	// *** WriteMissedRKCSVFile
	// ***************************************************	
	public void WriteMissedRKCSVFile(String fileName) {
		try {
			FileWriter fileWriter = new FileWriter(fileName);
			fileWriter.append(FILE_MISSED_RK_LIST_HEADER);
			
			for (RoutingKey next : routingKeysList) {
				String row = 	next.getRoutingKeyName() + CSV_SEPARATOR
								+ next.getMissingSequenceNo() + CSV_SEPARATOR
								+ next.isbReceived() + CSV_SEPARATOR
								+ next.getMissingSinceDateTime() + CSV_SEPARATOR
								+ next.getReceivedFinallyAtDateTime() + CSV_SEPARATOR
								+ next.getTimeDiffSec() +  NEW_LINE_SEPARATOR;
				fileWriter.append(row);
			}			
			fileWriter.flush();
			fileWriter.close();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}
	
	// ***************************************************
	// *** Display one RK in the GUI
	// ***************************************************	
	public String Display(GUIWindow guiWindow) {
		String s=RetrieveRKList();
		guiWindow.AddText(s);
		return s;
	}
	// ***************************************************
	// *** Return a String with the content of the RK List
	// ***************************************************	
	public String RetrieveRKList() {
		
		String s="";
		s = "********** Order Exe Report RKs ***********";
		int RKNb = 0;
		// start by listing Order Exe Report ".bg." Routing Keys: 
		for (RoutingKey next : routingKeysList) {
			if (next.getRoutingKeyName().contains("6_0.bg.")) {
				RKNb++;
				s = s + "\n* \tRouting Key #" + RKNb + ": "+ next.Display();
			}
		}
		s = s+ "\n";
		s = s+ "********** Other broadcasts RKs ***********";
		
		// list the rest
		for (RoutingKey next : routingKeysList) {
			if (!next.getRoutingKeyName().contains("6_0.bg.")) {
				RKNb++;
				s = s + "\n* \tRouting Key #" + RKNb + ": "+ next.Display();
			}
		}
		
		
		s = s+ "\n**************** END of Routing Keys ****************";
		return s;
	}

	// ***************************************************
	// *** Add a routing key to the list 
	// ***************************************************
	public int AddRoutingKey(String routingKeyName, int previousSequenceNo, int currentSequenceNo, int nbOfBroadcasts, String msgType) {
		RoutingKey routingKey = new RoutingKey();
		routingKey.setRoutingKeyName(routingKeyName);
		routingKey.setCurrentSequenceNo(currentSequenceNo);
		routingKey.setPreviousSequenceNo(previousSequenceNo);
		routingKey.setNbOfBroadcasts(nbOfBroadcasts);
		routingKey.setCurrentBroadcastMsgType(msgType);
		routingKeysList.add(routingKey);
		return routingKeysList.size();
	}
	
	// ***************************************************
	// *** AddMissingRoutingKey (same object but different attributes used)
	// ***************************************************
	public int AddMissingRoutingKey(String routingKeyName, int missingSequenceNo) {
				
		RoutingKey routingKey = new RoutingKey();
		routingKey.setRoutingKeyName(routingKeyName);
		routingKey.setMissingSequenceNo(missingSequenceNo);
		routingKey.setbReceived(false);
		routingKey.setMissingSinceDateTime(sdf.format(new Date()));
		routingKeysList.add(routingKey);
		
		WriteMissedRKCSVFile(FILE_MISSED_RK_LIST_NAME);
		this.DisplayMissingRoutingKey();
		return routingKeysList.size();
	}
	
	// ***************************************************
	// *** Display the List of Missing RK
	// ***************************************************
	public String DisplayMissingRoutingKey() {
		String s= FILE_MISSED_RK_LIST_HEADER; 	
		
		if (this.routingKeysList.size() > 0) {
			for (RoutingKey next : this.routingKeysList) {
				s = s +	String.format("%1$-" + 60 + "s",next.getRoutingKeyName()) + CSV_SEPARATOR
						+ TABULATION  + TABULATION+ TABULATION+ next.getMissingSequenceNo() + CSV_SEPARATOR
						+ TABULATION +next.isbReceived() + CSV_SEPARATOR
						+ TABULATION +next.getMissingSinceDateTime() + CSV_SEPARATOR
						+ TABULATION +next.getReceivedFinallyAtDateTime() + CSV_SEPARATOR
						+ TABULATION +next.getTimeDiffSec() +  NEW_LINE_SEPARATOR;
			}
		}
		return s;
	}
	// ***************************************************
	// *** UpdateMissingSeqNo (for RK for which there was a gap)
	// ***************************************************
	public int UpdateMissingSeqNo(String routingKeyName, int receivedSequenceNo) {
				
		for (RoutingKey next : this.routingKeysList) {
			if ( next.getRoutingKeyName().contentEquals(routingKeyName) 
						&& next.getMissingSequenceNo() == receivedSequenceNo) {

				next.setCurrentSequenceNo(receivedSequenceNo);
				next.setbReceived(true);
				next.setReceivedFinallyAtDateTime(sdf.format(new Date()));
				try {
				// parse method is used to parse the text from a string to produce the date
					Date d1 = sdf.parse(next.getMissingSinceDateTime());
					Date d2 = sdf.parse(next.getReceivedFinallyAtDateTime());
		  
		            // Calculate time difference in milliseconds
		            long differenceInTime = (d2.getTime() - d1.getTime()) / 1000;
		            next.setTimeDiffSec(String.valueOf(differenceInTime));
				} catch (ParseException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		WriteMissedRKCSVFile(FILE_MISSED_RK_LIST_NAME);
		this.DisplayMissingRoutingKey();
		return routingKeysList.size();
	}
	
	// ****************************************************************
	// *** update a routing key in the list with received seq nb values
	// ***************************************************************
	public int UpdateRoutingKeysList (String currentMsgRoutingKeyName, int currentMsgSequenceNo, String currentMsgType,RoutingKeysList missingRoutingKeysList, ControlBoard controlBoardFrame) {
		boolean routingKeyFound = false;
		
		// gap detection mechanism
		//guiWindowRKProcessing.AddText("new seq nb for RK = '" + currentMsgRoutingKeyName + "' Current SeqNo ='"+ currentMsgSequenceNo+"' type=" + currentMsgType);
		
		// if at least one gap has already been detected, update the list of Missing Seq No for the RK that are already in the list of RK that experienced a missing broadcast
		if (nbGaps > 0 ) {
			missingRoutingKeysList.UpdateMissingSeqNo(currentMsgRoutingKeyName, currentMsgSequenceNo);
		}
		else
		{
			// delete the missing Seq No List file
			File f = new File(FILE_MISSED_RK_LIST_NAME);
			if (f.exists()) {
				f.delete();
			  }
		}

		// go through the RK list until current RK is found
		for (RoutingKey next : this.routingKeysList) {

			// look for a Routing Key with the same name in the list
			if (next.getRoutingKeyName().contentEquals(currentMsgRoutingKeyName)  ) {

				int previousSeqNo = next.getCurrentSequenceNo();
				String previousMsgType = next.getCurrentBroadcastMsgType();
				int expectedSeqNo = previousSeqNo+1; // still equal to the previous seq No at that stage ; = 1 means that previous seq No = 0, meaning that
				int receivedSeqNo = currentMsgSequenceNo;
				int missingSeqNo = next.getMissingSequenceNo(); // get the  last missing seq no (if any ; if none : = 0)
				int nbOfBroadcastsForThatRK = next.getNbOfBroadcasts()+1; // = 1 if the 1st one for that RK

				// archive current values into "previous" values
				next.setPreviousBroadcastMsgType(previousMsgType);
				next.setPreviousSequenceNo(previousSeqNo);
				
				// update the RK list with current msg info
				next.setNbOfBroadcasts(nbOfBroadcastsForThatRK);
				next.setCurrentSequenceNo(receivedSeqNo);
				
				routingKeyFound = true;
				//guiWindowRKProcessing.AddText(next.getRoutingKeyName() + ": RK already received / exists in the list");
								
				// raise a gap if sequence No is not continuous for a given routing key, as of the 2nd broadcasts for that RK (otherwise there is no reference yet, therefor no "Expected" seq No
				if (nbOfBroadcastsForThatRK > 1 && expectedSeqNo != receivedSeqNo ) {
					nbGaps++;
					// update RK list with gap info
					next.setGapDetected(true);
					next.setMissingSequenceNo(expectedSeqNo);
					missingRoutingKeysList.AddMissingRoutingKey(currentMsgRoutingKeyName, expectedSeqNo);  
					
					// update the GUI gap window
					String gapMsg = "GAP DETECTED #" + nbGaps + "; RK=" + next.getRoutingKeyName() 
					+ " ; expected : " + expectedSeqNo  
					+ " ; received : " + receivedSeqNo + ";(gap of +" + (receivedSeqNo-expectedSeqNo) + ")"
					+ " ; missing : " + expectedSeqNo
					+ " ; nb of broadcasts : " + nbOfBroadcastsForThatRK
					+ " ; triggering msg type = " + next.getCurrentBroadcastMsgType()
					+ " ; previous msg type = " + next.getPreviousBroadcastMsgType();
				//	controlBoardFrame.textAreaGaps.append(gapMsg);
					// here the app should implement a gap recovery mechanism: 
					// 1. wait a couple of seconds to see if the missing broadcast with the right sequence number arrives
					// 2a. if it does not : send inquiry requests (without necessarily logging out) to "fill in what's missing" (please check the M7 API FAQ document for more details)
					// 2b. if it does : process broadcasts for that RK in the sequence nb order
				}
				// if not a gap, mention if the missing Sequence No finally arrived later
				else if (missingSeqNo != 0 && missingSeqNo == receivedSeqNo ) {
					nbGaps++;
					next.setGapDetected(true);
					String gapMsg = "GAP FILLED " + nbGaps + ": RK=" + next.getRoutingKeyName() 
					+ " : missing : " + missingSeqNo  
					+ " received : " + receivedSeqNo  
					+ " triggering msg type = " + next.getCurrentBroadcastMsgType();
					
					controlBoardFrame.textAreaGaps.append(gapMsg);
				}
				// display the List of RK
				// guiWindowRKList.AddTextStaticPlace(this.RetrieveRKList());
				
				WriteFileRKList(FILE_RK_LIST_NAME);
				
				break;
			}
		}
		if (routingKeyFound == false) {
			// the RK does not exist yet in the list (no broadcast with that RK received so far) : add it to the list
			System.out.println(sdf.format(new Date()) + " RK not found : added to the list");
			// convention to initialize previousSeqNo with the current one (to avoid setting "0")
			this.AddRoutingKey(currentMsgRoutingKeyName, currentMsgSequenceNo, currentMsgSequenceNo, 1, currentMsgType);
			
			updateGUITable(controlBoardFrame.RK_Gaps_GUI_Panel);
			WriteFileRKList(FILE_RK_LIST_NAME);
		}

		return nbGaps;
		
	}

	// ************************************************************************
	// *** Update the broadcast GUI table 
	// ************************************************************************
	public void updateGUITable(GUI_Panel RK_Gaps_GUI_Panel) {
		int row = 0;
		int columnNb = 0;
		
		JTable table = RK_Gaps_GUI_Panel.table;
		JTextField commentText = RK_Gaps_GUI_Panel.commentText;
		DefaultTableModel dataModel = RK_Gaps_GUI_Panel.dataModel;
		
		commentText.setText("Nb of RK = " + this.routingKeysList.size() + " Nb of gaps = " + this.nbGaps);

	  for (RoutingKey next : routingKeysList) {
		  if  (row == table.getRowCount()) dataModel.addRow(new Object[] { row });
	  
		  columnNb = 0; // {"#","Reception Time", "Sending Time","Type", "Routing Key",	  "Sequence No", "XML payload"};
		  table.setValueAt(row+1,row,columnNb); columnNb++ ;
		  table.setValueAt(next.getRoutingKeyName(),row,columnNb); columnNb++;
		  table.setValueAt(next.getCurrentSequenceNo(),row, columnNb); columnNb++ ;
		  table.setValueAt(next.GetGapDetected(),row,columnNb); columnNb++ ;
		  table.setValueAt(next.getMissingSequenceNo(),row,columnNb); columnNb++ ;
		  table.setValueAt(next.getMissingSinceDateTime(),row,columnNb); columnNb++ ;

		  row++;
	  }
	}
	
}
